//////////////////////////////////// Memo24 /////////////////////////////////////
////////////// v1.32  2004, March 24th by Johannes Wallroth, Germany////////////
////////////////////////////// www.programming.de ///////////////////////////////
//Creator-ID: D81z

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;
import waba.util.*;

public class Memo24 extends MainWindow 
{		
	static final int SIZE = 60;
	static final int ROWS = 5;
	static final int MAX = 24;
	static final int offsetX = 10;
	static final int offsetY = 10;
	static final int COVER_UP_TIME = 1000;
	
	private String col0[] = {"Memo24", "New Game", "How to play", "Info about..."};	
	private MenuBar mbar;
	private int clickedTile, startTime;
	private boolean allowPopup = true, intro = true, gameOver, shuttingDown;		
	private Image theme;	
	private Grid grid;
	private Timer timer;
	private Font font;
	private FontMetrics fm;	
	private final Color YELLOW, GREEN1, GREEN2, RED;
	
	public Memo24() 
	{
  		highResPrepared = true;  		
  		setDoubleBuffer(true);    		
  		Settings.setPalmOSStyle(true);  
  		mbar = new MenuBar(new String[][]{col0});  		
  		mbar.setBackForeColors(new Color(153,153,255), Color.WHITE);
  		mbar.setPopColors(Color.WHITE, new Color(51,51,255), new Color(153,153,255));
  		setMenuBar(mbar);  	  		
  		font = defaultFont;	
  		fm = getFontMetrics(font);
  		startTime = Vm.getTimeStamp();
  		timer = addTimer(100);	  		 		
  		GREEN1 = new Color(51,102,51);	  		
  		GREEN2 = new Color(51,153,51);	  		
  		YELLOW = new Color(255,255,0);    		 		
  		RED = new Color(255,0,0);  		
  	}
  	  	
  	private void exitScreen()
  	{  		
  		MessageBox mb = new MessageBox("Error", "  High resolution display | required for this game! || Shutting down...  ", null);	
  		mb.setBackForeColors(new Color(255,255,0), new Color(255,0,0));	  		
		mb.makeUnmovable();			
  		mb.setUnpopDelay(5000);		  		
   		popupBlockingModal(mb);   	   		   		
   		exit(0);   		
  	}  	
  		
	public void onEvent(Event event) 
	{	
		if (event.type == PenEvent.PEN_DOWN)
		{
			PenEvent pe = (PenEvent)event;
			
			//process only clicks within the tile area			
			Rect includeRect = new Rect(offsetX, offsetY, SIZE*ROWS, SIZE*ROWS);
			//ignore the tile in the center
			Rect excludeRect = new Rect(SIZE*2 + offsetX, SIZE*2 + offsetY, SIZE, SIZE);
		
			if(includeRect.contains(pe.x, pe.y) && !excludeRect.contains(pe.x, pe.y) && !gameOver && !intro)
			{						
				clickedTile = grid.clickedBoard(pe.x - offsetX, pe.y - offsetY);
							
				if(grid.allCorrect() && allowPopup)							
				{
					allowPopup = false;
					gameOver = true;					
					timer = addTimer(1000);		
				}
				else
				{
					if(grid.triggerCoverUp())	
					{
						removeTimer(timer);			
						timer = addTimer(COVER_UP_TIME);	
					}
					else
						removeTimer(timer);								
				}
			}
   			   			
   			repaintNow();
		}
		
		if (event.type == ControlEvent.TIMER)	
		{		
			int introSeconds = 2;	//minimum intro length
			
			if(intro)
			{		
				if(Settings.screenWidth < 320)	 
				{
					shuttingDown = true;
					repaintNow();
					removeTimer(timer);
					exitScreen();
				}
				else
					if(Vm.getTimeStamp() - startTime >= introSeconds*1000)
					{			
						theme = new Image("Images.bmp");	
			
						//image has been loaded
						if(theme.getWidth() > 0)
						{		
							removeTimer(timer);
							grid = new Grid();			
							intro = false;					
						}					
						repaintNow();
					}
			}			
			else if(gameOver)
			{		
				removeTimer(timer);					
				MessageBox mb = new MessageBox("Game over", 
					" | Cleared in " + grid.getClicks() + " moves! | ", null);
			
				mb.setBackColor(new Color(0,0,255));		
				mb.setUnpopDelay(4000);	
				mb.makeUnmovable();
   				popupBlockingModal(mb);   	   			
   				coverSlowly();		   				
	   			grid = new Grid(); 			 	
   				allowPopup = true;  
   				gameOver = false; 	
   				repaintNow();		  	   	   			   						
   			}  
   			else
   			{
   				removeTimer(timer);
   				grid.coverTiles(-1);
   				repaintNow();
   			} 	   					
		}
		
		if (event.type == ControlEvent.WINDOW_CLOSED && event.target == mbar)		
		{			
			if(mbar.getSelectedMenuItem() == 1)				
				grid = new Grid(); 
				
			if(mbar.getSelectedMenuItem() == 2)
			{	
				String helptext = "Your task is to uncover all 24 tiles, which surround the static center tile in as few moves as possible. Simply tap on a tile to uncover it - remember the picture - and tap the next one. If the images match, they remain uncovered - if they don't, they are covered again after a second or when you click next time (you can click the same tile again, if you wish). The number of moves is displayed on the center tile. When the game is solved, a score popup window appears - after that, a new game starts automatically.";												
				helptext = Convert.insertLineBreak(Settings.screenWidth-6, '|', fm, helptext);							
				MessageBox mb = new MessageBox("How to play", helptext);	
				mb.setBackForeColors(Color.WHITE, new Color(0,153,0));	
				mb.setFont(font);				
				mb.makeUnmovable();
				mb.setTextAlignment(LEFT);
				popupBlockingModal(mb);							
			}
						
			if(mbar.getSelectedMenuItem() == 3)
			{				
				MessageBox mb = new MessageBox("Memo24", "  Johannes Wallroth | www.programming.de ");	
				mb.setBackForeColors(Color.WHITE, new Color(0,153,0));	
				mb.makeUnmovable();
				popupBlockingModal(mb);							
			}	
			repaintNow();	
		}
	}	
	
	public void coverSlowly()
	{
		for(int i=0; i<MAX; i++)		
		{			
			if(i != 12)
			{
				Vm.sleep(50);	
				grid.cover(i);
			   	repaintNow();						
			 }				   				   		
		 }
		 Vm.sleep(50);	
		 grid.cover(12);
		 repaintNow();	
	}

	public void onPaint(Graphics g) 
	{		
		if(intro)
		{
			g.setBackColor(GREEN1);
			g.clearScreen();
			
			if(!shuttingDown)
			{
				g.setForeColor(GREEN2);
			
				for(int x=15; x<300; x+=60)
					for(int y=15; y<300; y+=60)
						g.drawRect(x,y,50,50);
			
				Font bigFont = new Font(font.getName(), Font.BOLD, 14);
				g.setFont(bigFont);

				g.setForeColor(YELLOW);	
				g.drawText("Memo24", 118, 85);	
			
				g.setFont(font);
				g.setForeColor(Color.WHITE);				
				g.drawText(" Johannes Wallroth", 73, 198);		
				g.drawText("www.programming.de", 73, 220);		
			}
		}	
		else
		{
			//The background
			if(gameOver)
				g.setBackColor(RED);
			else
				g.setBackColor(119,119,119);
				
			g.clearScreen();
		
			for (int i=0; i<MAX; i++)
			{				
				Tile tile = grid.getTile(i);
				Coord coord = tile.getCoord();
				int imgIndex = tile.getImageIndex();
				
				if(tile.isOpen())
					g.copyRect(theme, SIZE*imgIndex, 0, SIZE, SIZE, coord.x + offsetX, coord.y + offsetY);			
				else	//cover tile			
					g.copyRect(theme, SIZE*12, 0, SIZE, SIZE, coord.x + offsetX, coord.y + offsetY);
				
				//center tile
				g.copyRect(theme, SIZE*13, 0, SIZE, SIZE, SIZE*2 + offsetX, SIZE*2 + offsetY);
			}
		
			//center the score text	 		
 			String s = Convert.toString(grid.getClicks());
 			int stringWidth = fm.getTextWidth(s);
 		
			g.setForeColor(Color.WHITE);
		
			if(grid.getClicks() > 0)
				g.drawText(s, 161 - stringWidth/2, 149);		
		}		
	}  	
}
